@@ -19,7 +19,6 @@
 #include "EphyWrapper.h"
 #include "GlobalHistory.h"
 #include "ProgressListener.h"
-#include "PrintProgressListener.h"
 #include "ephy-embed.h"
 #include "ephy-string.h"
 
@@ -193,8 +192,7 @@ nsresult EphyWrapper::Print (nsIPrintSettings *options, PRBool preview)
 
 	if (!preview)
 	{
-		GPrintListener *listener = new GPrintListener();
-		result = print->Print (options, listener);
+		result = print->Print (options, nsnull);
 	}
 	else
 	{
@@ -213,7 +211,7 @@ nsresult EphyWrapper::PrintPreviewClose (void)
 	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
 
 	rv = print->GetDoingPrintPreview(&isPreview);
-	if (isPreview == PR_TRUE)
+	if (NS_SUCCEEDED (rv) && isPreview == PR_TRUE)
 	{
 		rv = print->ExitPrintPreview();
 	}
@@ -541,21 +539,6 @@ nsresult EphyWrapper::GetWebNavigation(nsIWebNavigation **aWebNavigation)
 	return NS_OK;
 }
 
-nsresult EphyWrapper::ReloadDocument ()
-{
-	nsresult result;
-
-	nsCOMPtr<nsIWebNavigation> wn;
-	result = GetWebNavigation(getter_AddRefs(wn));
-	if (!wn || !NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
-
-	result = wn->Reload (nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
-			     nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY);
-	if (!NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
-
-	return NS_OK;
-}
-
 nsresult EphyWrapper::LoadDocument(nsISupports *aPageDescriptor,
 				     PRUint32 aDisplayType)
 {
