@@ -19,6 +19,7 @@
 #include "egg-toolbars-model.h"
 #include "eggmarshalers.h"
 
+#include <unistd.h>
 #include <string.h>
 #include <libxml/tree.h>
 #include <gdk/gdkproperty.h>
@@ -126,6 +127,7 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
 	      name = egg_toolbars_model_get_item_name (t, item->type, item->id);
 	      xmlSetProp (node, "type", item->type);
 	      xmlSetProp (node, "name", name);
+	      g_free (name);
 	    }
 	}
     }
@@ -312,21 +314,21 @@ egg_toolbars_model_add_separator (EggToolbarsModel *t,
 		 toolbar_position, real_position);
 }
 
-void
-egg_toolbars_model_add_item (EggToolbarsModel    *t,
-			     int		  toolbar_position,
-			     int		  position,
-			     const char          *id,
-			     const char          *type)
+gboolean
+impl_add_item (EggToolbarsModel    *t,
+	       int		    toolbar_position,
+	       int		    position,
+	       const char          *id,
+	       const char          *type)
 {
   GNode *parent_node;
   GNode *node;
   EggToolbarsItem *item;
   int real_position;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
-  g_return_if_fail (id != NULL);
-  g_return_if_fail (type != NULL);
+  g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), FALSE);
+  g_return_val_if_fail (id != NULL, FALSE);
+  g_return_val_if_fail (type != NULL, FALSE);
 
   parent_node = g_node_nth_child (t->priv->toolbars, toolbar_position);
   item = toolbars_item_new (id, type, FALSE);
@@ -337,6 +339,8 @@ egg_toolbars_model_add_item (EggToolbarsModel    *t,
 
   g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[ITEM_ADDED], 0,
 		 toolbar_position, real_position);
+
+  return TRUE;
 }
 
 static void
@@ -484,6 +488,7 @@ egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 
   object_class->finalize = egg_toolbars_model_finalize;
 
+  klass->add_item = impl_add_item;
   klass->get_item_id = impl_get_item_id;
   klass->get_item_name = impl_get_item_name;
   klass->get_item_type = impl_get_item_type;
@@ -663,6 +668,17 @@ egg_toolbars_model_toolbar_nth (EggToolbarsModel *t,
   return tdata->name;
 }
 
+gboolean
+egg_toolbars_model_add_item (EggToolbarsModel *t,
+			     int	       toolbar_position,
+			     int               position,
+			     const char       *id,
+			     const char       *type)
+{
+  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
+  return klass->add_item (t, toolbar_position, position, id, type);
+}
+
 char *
 egg_toolbars_model_get_item_id (EggToolbarsModel *t,
 			        const char       *type,
