@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.21 2003/08/11 12:45:43 mpeseng Exp $
+ *  $Id: ephy-node-view.c,v 1.25 2003/08/22 11:02:08 marco Exp $
  */
 
 #include <gtk/gtktreeselection.h>
@@ -69,10 +69,16 @@ struct EphyNodeViewPrivate
 
 	int searchable_data_column;
 
+	gboolean drag_started;
 	int drag_button;
 	int drag_x;
         int drag_y;
 	GtkTargetList *source_target_list;
+
+	gboolean drop_occurred;                                                                                          
+	gboolean have_drag_data;
+	GtkSelectionData *drag_data;
+	guint scroll_id;
 };
 
 enum
@@ -91,6 +97,8 @@ enum
 	PROP_FILTER
 };
 
+#define AUTO_SCROLL_MARGIN 20
+
 static GObjectClass *parent_class = NULL;
 
 static guint ephy_node_view_signals[LAST_SIGNAL] = { 0 };
@@ -223,6 +231,8 @@ get_node_from_path (EphyNodeView *view, GtkTreePath *path)
 	EphyNode *node;
 	GtkTreeIter iter, iter2;
 
+	if (path == NULL) return NULL;
+
 	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
 	gtk_tree_model_sort_convert_iter_to_child_iter
 		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
@@ -233,103 +243,268 @@ get_node_from_path (EphyNodeView *view, GtkTreePath *path)
 	return node;
 }
 
-static gboolean
-drag_motion_cb (GtkWidget *widget,
-                GdkDragContext *context,
-                gint x,
-                gint y,
-                guint time,
-	        EphyNodeView *view)
+static void
+gtk_tree_view_vertical_autoscroll (GtkTreeView *tree_view)
 {
-	EphyNode *node;
-	GtkTreePath *path = NULL;
-	GtkTreeViewDropPosition pos;
-	gboolean res;
-	EphyNodeViewPriority priority;
-
-	g_signal_stop_emission_by_name (widget, "drag_motion");
-
-	res = gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
-					         x, y, &path, &pos);
-	if (!res) return TRUE;
-
-	node = get_node_from_path (view, path);
-
-	priority = ephy_node_get_property_int (node, view->priv->priority_prop_id);
-
-	if (priority != EPHY_NODE_VIEW_ALL_PRIORITY &&
-	    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY)
+	GdkRectangle visible_rect;
+	GtkAdjustment *vadjustment;
+	GdkWindow *window;
+	int y;
+	int offset;
+	float value;
+	
+	window = gtk_tree_view_get_bin_window (tree_view);
+	vadjustment = gtk_tree_view_get_vadjustment (tree_view);
+	
+	gdk_window_get_pointer (window, NULL, &y, NULL);
+	
+	y += vadjustment->value;
 
+	gtk_tree_view_get_visible_rect (tree_view, &visible_rect);
+	
+	offset = y - (visible_rect.y + 2 * AUTO_SCROLL_MARGIN);
+	if (offset > 0)
 	{
-		gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (widget), path,
-						 GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
-		gdk_drag_status (context, context->suggested_action, time);
+		offset = y - (visible_rect.y + visible_rect.height - 2 * AUTO_SCROLL_MARGIN);
+		if (offset < 0)
+		{
+			return;
+		}
+	}
+
+	value = CLAMP (vadjustment->value + offset, 0.0,
+		       vadjustment->upper - vadjustment->page_size);
+	gtk_adjustment_set_value (vadjustment, value);
+}
+
+static int
+scroll_timeout (gpointer data)
+{
+	GtkTreeView *tree_view = GTK_TREE_VIEW (data);
+	
+	gtk_tree_view_vertical_autoscroll (tree_view);
+
+	return TRUE;
+}
+
+static void
+remove_scroll_timeout (EphyNodeView *view)
+{
+	if (view->priv->scroll_id)
+	{
+		g_source_remove (view->priv->scroll_id);
+		view->priv->scroll_id = 0;
+	}
+}
+
+static void
+set_drag_dest_row (EphyNodeView *view,
+		   GtkTreePath *path)
+{
+	if (path)
+	{
+		gtk_tree_view_set_drag_dest_row
+			(GTK_TREE_VIEW (view),
+			 path,
+			 GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
 	}
 	else
 	{
-		gdk_drag_status (context, 0, time);
+		gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (view), 
+						 NULL, 
+						 0);
 	}
-
-	gtk_tree_path_free (path);
-
-	return TRUE;
 }
 
-static gboolean
-drag_drop_cb (GtkWidget *widget,
-              GdkDragContext *context,
-              gint x,
-              gint y,
-              guint time,
-	      EphyNodeView *view)
+static void
+clear_drag_dest_row (EphyNodeView *view)
+{
+	gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (view), NULL, 0);
+}
+
+static void
+get_drag_data (EphyNodeView *view,
+	       GdkDragContext *context, 
+	       guint32 time)
 {
 	GdkAtom target;
+	
+	target = gtk_drag_dest_find_target (GTK_WIDGET (view), 
+					    context, 
+					    NULL);
 
-	g_signal_stop_emission_by_name (widget, "drag_drop");
+	gtk_drag_get_data (GTK_WIDGET (view),
+			   context, target, time);
+}
 
-	target = gtk_drag_dest_find_target (widget, context,
-					    view->priv->drag_targets);
+static void
+free_drag_data (EphyNodeView *view)
+{
+	view->priv->have_drag_data = FALSE;
 
-	if (target != GDK_NONE)
+	if (view->priv->drag_data)
 	{
-		gtk_drag_get_data (widget, context, target, time);
+		gtk_selection_data_free (view->priv->drag_data);
+		view->priv->drag_data = NULL;
 	}
+}
+
+static gboolean
+drag_motion_cb (GtkWidget *widget,
+		GdkDragContext *context,
+		int x,
+		int y,
+		guint32 time,
+		EphyNodeView *view)
+{
+	EphyNode *node;
+	GtkTreePath *path;
+	GtkTreeViewDropPosition pos;
+	guint action = 0;
+	int priority;
+
+	gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
+					   x, y, &path, &pos);
+	
+	if (!view->priv->have_drag_data)
+	{
+		get_drag_data (view, context, time);
+	}
+
+	node = get_node_from_path (view, path);
+
+	if (node)
+	{
+		priority = ephy_node_get_property_int
+				(node, view->priv->priority_prop_id);
+
+		if (priority != EPHY_NODE_VIEW_ALL_PRIORITY &&
+		    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY)
+		{
+			action = context->suggested_action;
+		}
+	}
+	
+	if (action)
+	{
+		set_drag_dest_row (view, path);
+	}
+	else
+	{
+		clear_drag_dest_row (view);
+	}
+	
+	if (path)
+	{
+		gtk_tree_path_free (path);
+	}
+	
+	if (view->priv->scroll_id == 0)
+	{
+		view->priv->scroll_id = 
+			g_timeout_add (150, 
+				       scroll_timeout, 
+				       GTK_TREE_VIEW (view));
+	}
+
+	gdk_drag_status (context, action, time);
 
 	return TRUE;
 }
 
-static gboolean
-drag_data_received_cb (GtkWidget *widget,
-                       GdkDragContext *context,
-                       gint x,
-                       gint y,
-	               GtkSelectionData *selection_data,
-                       guint info,
-                       guint time,
-	               EphyNodeView *view)
+static void
+drag_leave_cb (GtkWidget *widget,
+	       GdkDragContext *context,
+	       guint32 time,
+	       EphyNodeView *view)
 {
-	GtkTreePath *path = NULL;
+	clear_drag_dest_row (view);
+
+	free_drag_data (view);
+
+	remove_scroll_timeout (view);
+}
+
+static void
+drag_data_received_cb (GtkWidget *widget,
+		       GdkDragContext *context,
+		       int x,
+		       int y,
+		       GtkSelectionData *selection_data,
+		       guint info,
+		       guint32 time,
+		       EphyNodeView *view)
+{
+	GtkTreePath *path;
 	GtkTreeViewDropPosition pos;
+	gboolean on_row;
 
-	g_signal_stop_emission_by_name (widget, "drag_data_received");
+	if (selection_data->length <= 0 || selection_data->data == NULL)
+	{
+		return;
+	}
+	
+	on_row = gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
+					            x, y, &path, &pos);
 
-	if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
-					       x, y, &path, &pos))
+	if (!view->priv->have_drag_data)
+	{
+		view->priv->have_drag_data = TRUE;
+		view->priv->drag_data = 
+			gtk_selection_data_copy (selection_data);
+	}
+
+	if (view->priv->drop_occurred)
 	{
 		EphyNode *node;
 		GList *uris;
+		gboolean success = FALSE;
+
+		g_return_if_fail (on_row && path != NULL);
 
 		node = get_node_from_path (view, path);
 
 		uris = gnome_vfs_uri_list_parse (selection_data->data);
-		g_signal_emit (G_OBJECT (view),
-			       ephy_node_view_signals[NODE_DROPPED], 0,
-			       node, uris);
-		gnome_vfs_uri_list_free (uris);
+	
+		if (uris != NULL)
+		{
+			/* FIXME fill success */
+			g_signal_emit (G_OBJECT (view),
+				       ephy_node_view_signals[NODE_DROPPED], 0,
+				       node, uris);
+			gnome_vfs_uri_list_free (uris);
+		}
 
+		view->priv->drop_occurred = FALSE;
+		free_drag_data (view);
+		gtk_drag_finish (context, success, FALSE, time);
+	}
+
+	if (path)
+	{
 		gtk_tree_path_free (path);
 	}
 
+	/* appease GtkTreeView by preventing its drag_data_receive
+	 * from being called */
+	g_signal_stop_emission_by_name (GTK_TREE_VIEW (view),
+					"drag_data_received");
+}
+
+static gboolean
+drag_drop_cb (GtkWidget *widget,
+	      GdkDragContext *context,
+	      int x, 
+	      int y,
+	      guint32 time,
+	      EphyNodeView *view)
+{
+	view->priv->drop_occurred = TRUE;
+
+	get_drag_data (view, context, time);
+	remove_scroll_timeout (view);
+	clear_drag_dest_row (view);
+	
 	return TRUE;
 }
 
@@ -355,6 +530,8 @@ ephy_node_view_enable_drag_dest (EphyNodeView *view,
 			  G_CALLBACK (drag_drop_cb), view);
 	g_signal_connect (treeview, "drag_motion",
 			  G_CALLBACK (drag_motion_cb), view);
+	g_signal_connect (treeview, "drag_leave",
+			  G_CALLBACK (drag_leave_cb), view);
 }
 
 static void
@@ -538,6 +715,38 @@ stop_drag_check (EphyNodeView *view)
         view->priv->drag_button = 0;
 }
 
+static gboolean
+button_event_modifies_selection (GdkEventButton *event)
+{
+	return (event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) != 0;
+}
+
+static void
+did_not_drag (EphyNodeView *view,
+              GdkEventButton *event)
+{
+        GtkTreeView *tree_view;
+        GtkTreeSelection *selection;
+        GtkTreePath *path;
+                                                                                                                              
+        tree_view = GTK_TREE_VIEW (view);
+        selection = gtk_tree_view_get_selection (tree_view);
+                                                                                                                              
+        if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
+                                           &path, NULL, NULL, NULL))
+	{
+                if((event->button == 1 || event->button == 2) &&
+                   gtk_tree_selection_path_is_selected (selection, path) &&
+                   !button_event_modifies_selection (event))
+		{
+                        gtk_tree_selection_unselect_all (selection);
+                        gtk_tree_selection_select_path (selection, path);
+                }
+                                                                                                                              
+                gtk_tree_path_free (path);
+        }                                                                                                              
+}
+
 static void
 drag_data_get_cb (GtkWidget *widget,
                   GdkDragContext *context,
@@ -577,6 +786,11 @@ button_release_cb (GtkWidget *widget,
         if (event->button == view->priv->drag_button)
 	{
 		stop_drag_check (view);
+		if (!view->priv->drag_started)
+		{
+                        did_not_drag (view, event);
+                        return TRUE;
+                }
         }
         return FALSE;
 }
@@ -606,6 +820,7 @@ motion_notify_cb (GtkWidget *widget,
                                  (GdkEvent*)event);
 
 			stop_drag_check (view);
+			view->priv->drag_started = TRUE;
 
 			ref_list = get_selection_refs (GTK_TREE_VIEW (widget));
 			g_object_set_data_full (G_OBJECT (context),
@@ -620,12 +835,6 @@ motion_notify_cb (GtkWidget *widget,
 	return TRUE;
 }
 
-static gboolean
-button_event_modifies_selection (GdkEventButton *event)
-{
-	return (event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) != 0;
-}
-
 static gboolean
 ephy_node_view_button_press_cb (GtkWidget *treeview,
 			        GdkEventButton *event,
@@ -672,6 +881,7 @@ ephy_node_view_button_press_cb (GtkWidget *treeview,
 		}
 		else if (event->button == 1)
 		{
+			view->priv->drag_started = FALSE;
 			view->priv->drag_button = event->button;
 	                view->priv->drag_x = event->x;
 	                view->priv->drag_y = event->y;
@@ -830,6 +1040,38 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	gtk_tree_path_free (path);
 }
 
+static int
+compare_string_values (const GValue *a_value, const GValue *b_value)
+{
+        const char *str1, *str2;
+        int retval;
+                                                                                                                              
+        str1 = g_value_get_string (a_value);
+        str2 = g_value_get_string (b_value);
+                                                                                                                              
+        if (str1 == NULL)
+        {
+                retval = -1;
+        }
+        else if (str2 == NULL)
+        {
+                retval = 1;
+        }
+        else
+        {
+	        char *str_a;
+	        char *str_b;
+
+                str_a = g_utf8_casefold (str1, -1);
+                str_b = g_utf8_casefold (str2, -1);
+                retval = g_utf8_collate (str_a, str_b);
+                g_free (str_a);
+                g_free (str_b);
+        }
+                                                                                                                              
+        return retval;
+}
+
 static int
 ephy_node_view_sort_func (GtkTreeModel *model,
 			  GtkTreeIter *a,
@@ -851,20 +1093,14 @@ ephy_node_view_sort_func (GtkTreeModel *model,
 		GType type = gtk_tree_model_get_column_type (model, column);
 		GValue a_value = {0, };
 		GValue b_value = {0, };
-		gchar *stra, *strb;
+
 		gtk_tree_model_get_value (model, a, column, &a_value);
 		gtk_tree_model_get_value (model, b, column, &b_value);
 
 		switch (G_TYPE_FUNDAMENTAL (type))
 		{
 		case G_TYPE_STRING:
-			/* FIXME: this is horribly inefficient */
-			stra = g_utf8_casefold (g_value_get_string (&a_value), -1);
-			strb = g_utf8_casefold (g_value_get_string (&b_value), -1);
-			g_return_val_if_fail (stra != NULL && strb != NULL, 0);
-			retval = g_utf8_collate (stra, strb);
-			g_free (stra);
-			g_free (strb);
+			retval = compare_string_values (&a_value, &b_value);
 			break;
 		case G_TYPE_INT:
 			if (g_value_get_int (&a_value) < g_value_get_int (&b_value))
